#-----------------------------------------------------------------------------
# Project    : Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File       : pnr.tcl
# Authors    : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab        : GRM - Polytechnique Montréal
# Date       : <2019-07-24 Wed>
#-----------------------------------------------------------------------------
# Description: Placement et routage du compteur BCD avec Innovus
# HOW-TO     : source setup.csh && cd implementation/pnr && innovus
#            : innovus -files ${SCRIPTS_DIR}/pnr.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT    1
set DO_FP      1
set DO_POWER   1
set DO_IO      1
set DO_PLACE   1
set DO_CTS     1
set DO_ROUTE   1
set DO_CHECK   1
set DO_SAVE    1

set DESIGN compteur

# Create directories if they do not exists
file mkdir $::env(PNR_REP_DIR) $::env(PNR_NET_DIR)

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT } {

    set init_oa_ref_lib    [list gsclib045_tech gsclib045 giolib045]
    set init_mmmc_file     $::env(CONST_DIR)/mmmc.tcl
    set init_verilog       $::env(SYN_NET_DIR)/${DESIGN}.syn.v
    set init_design_settop 1
    set init_top_cell      $DESIGN
    set init_gnd_net       VSS
    set init_pwr_net       VDD

    init_design
}

#-----------------------------------------------------------------------------
# FLOORPLAN
#-----------------------------------------------------------------------------
if { $DO_FP } {

    floorPlan -site CoreSite -r 0.9 0.6 1 1 1 1
}

#-----------------------------------------------------------------------------
# ALIMENTATIONS
#-----------------------------------------------------------------------------
if { $DO_POWER } {

    globalNetConnect VDD -type pgpin -pin VDD -inst * -override
    globalNetConnect VSS -type pgpin -pin VSS -inst * -override
    globalNetConnect VDD -type tiehi -inst * -override
    globalNetConnect VSS -type tielo -inst * -override

    addStripe -nets VDD           \
              -layer Metal1       \
              -direction vertical \
              -width 0.6          \
              -number_of_sets 1   \
              -start_from left    \
              -start_offset -0.8

    addStripe -nets VSS           \
              -layer Metal1       \
              -direction vertical \
              -width 0.6          \
              -number_of_sets 1   \
              -start_from right   \
              -start_offset -0.8

    sroute -nets { VDD VSS }                   \
           -connect { corePin floatingStripe } \
           -layerChangeRange { Metal1(1) Metal1(1) }
}

#-----------------------------------------------------------------------------
# I/O
#-----------------------------------------------------------------------------
if { $DO_IO } {

    # Nets
    set top_nets [list [get_nets -quiet "i_clk"]  \
                       [get_nets -quiet "i_rstn"] \
                       [get_nets -quiet "i_en"]]

    set bottom_nets [list [get_nets -quiet "o_cnt[*]"]]

    set top_n [list ]
    foreach nets $top_nets {foreach_in_collection n $nets {lappend top_n [get_object_name $n]}}

    set bottom_n [list ]
    foreach nets $bottom_nets {foreach_in_collection n $nets {lappend bottom_n [get_object_name $n]}}

    setPinAssignMode -pinEditInBatch true

    editPin -pin "${top_n}"	-side Top -layer 2           \
            -pinWidth 0.06 -pinDepth 0.335               \
            -spreadDirection clockwise -spreadType range \
            -start 2 9.12 -end 10 9.12

    editPin -pin "${bottom_n}" -side Bottom -layer 2            \
            -pinWidth 0.08 -pinDepth 0.25                       \
            -spreadDirection counterclockwise -spreadType range \
            -start 1 0 -end 11 0
}

#-----------------------------------------------------------------------------
# PLACEMENT
#-----------------------------------------------------------------------------
if { $DO_PLACE } {

    setDesignMode -process 45
    setPlaceMode -place_global_reorder_scan false
    deleteAllScanCells
    placeDesign
    report_timing > $::env(PNR_REP_DIR)/${DESIGN}.place.timing.rpt
}

#-----------------------------------------------------------------------------
# ARBRE D'HORLOGE
#-----------------------------------------------------------------------------
if { $DO_CTS } {

    optDesign -preCTS -outDir $::env(PNR_REP_DIR)/cts

    set_ccopt_property buffer_cells \
        [list CLKBUFX20 CLKBUFX16 CLKBUFX12 CLKBUFX8 CLKBUFX6 CLKBUFX4 CLKBUFX3 CLKBUFX2]

    set_ccopt_property inverter_cells \
        [list CLKINVX20 CLKINVX6 CLKINVX8 CLKINVX16 CLKINVX12 CLKINVX4 CLKINVX3 CLKINVX2 CLKINVX1]

    set_ccopt_property use_inverters true

    ccopt_design -cts -outDir $::env(PNR_REP_DIR)/cts
    optDesign -postCTS -outDir $::env(PNR_REP_DIR)/cts

    report_timing > $::env(PNR_REP_DIR)/${DESIGN}.cts.timing.rpt
}

#-----------------------------------------------------------------------------
# ROUTE
#-----------------------------------------------------------------------------
if { $DO_ROUTE } {

    setNanoRouteMode -quiet -routeWithTimingDriven true
    routeDesign -globalDetail

    addFiller -cell FILL32 FILL16 FILL8 FILL4 FILL2 FILL1 -prefix FILLER
}

#-----------------------------------------------------------------------------
# CHECK
#-----------------------------------------------------------------------------
if { $DO_CHECK } {

    # DRC
    set_verify_drc_mode -report $::env(PNR_REP_DIR)/${DESIGN}.drc.rpt
    verify_drc

    # Connectivity
    verifyConnectivity -type all -error 1000 -warning 50 -report $::env(PNR_REP_DIR)/${DESIGN}.con.rpt

    # Timing
    setAnalysisMode -analysisType onChipVariation
    reset_parasitics
    extractRC
    report_timing > $::env(PNR_REP_DIR)/${DESIGN}.timing.rpt
}

#-----------------------------------------------------------------------------
# SAUVEGARDES
#-----------------------------------------------------------------------------
if { $DO_SAVE } {

    if {! [file isdirectory $::env(PNR_OA_LIB)] } {
        createLib $::env(PNR_OA_LIB) -attachTech gsclib045_tech
    }

    saveDesign -cellview "$::env(PNR_OA_LIB) $DESIGN layout"

    saveNetlist $::env(PNR_NET_DIR)/${DESIGN}.pnr.v
    saveNetlist -phys $::env(PNR_NET_DIR)/${DESIGN}.pnr.phy.v

    write_sdf -view av_fast -target_application verilog -interconn noport          \
              -recrem split -setuphold split -adjust_setuphold_for_zero_hold_slack \
              $::env(PNR_NET_DIR)/${DESIGN}.pnr.sdf
}
