#-----------------------------------------------------------------------------
# Project    : Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File       : syn.tcl
# Authors    : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab        : GRM - Polytechnique Montréal
# Date       : <2019-07-24 Wed>
#-----------------------------------------------------------------------------
# Description: Script pour la synthèse du compteur BCD avec Genus
# HOW-TO     : > source setup.csh && cd implementation/syn
#            : > genus -files ${SCRIPTS_DIR}/syn.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT     1
set DO_ELAB     1
set DO_SDC      1
set DO_SYN      1
set DO_REPORTS  1

set DESIGN     compteur
set SYN_EFFORT high;    # low | medium | high | express

# Create directories if they do not exists
file mkdir $::env(SYN_REP_DIR) $::env(SYN_NET_DIR)

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT } {

    # Paramètres de base
    set_db / .information_level     9;    # Verbosité: 1-11
    set_db / .hdl_vhdl_read_version 2008; # Standard VHDL
    set_db / .max_cpus_per_server   4;    # Numbre max de thread utilisable par genus

    # Sources
    set_db / .init_hdl_search_path [list $::env(SRC_DIR)]

    # Librairies: Timing + Standard Cells + Interconnect
    set_db / .init_lib_search_path  [list $::env(FE_TIM_LIB) \
                                          $::env(BE_QRC_LIB) \
                                          $::env(BE_LEF_LIB) ]

    # Librairies "fast"
    read_libs          "fast_vdd1v0_basicCells.lib"
    read_physical -lef "gsclib045_tech.lef"
    read_qrc           "gpdk045.tch"

    set_db / .interconnect_mode ple; # wireload ou ple (Physical Layout Estimators)
}

#-----------------------------------------------------------------------------
# ELABORATION
#-----------------------------------------------------------------------------
set sources [list compteur.vhd]

if { $DO_ELAB } {

    # Paramètres de compilation
    set_db / .hdl_error_on_blackbox true
    set_db / .hdl_error_on_latch true

    # Compilation
    foreach src $sources {
       read_hdl -vhdl $src
    }

    # Élaboration
    elaborate $DESIGN

    # Vérifications
    check_design -all
}

#-----------------------------------------------------------------------------
# CONTRAINTES
#-----------------------------------------------------------------------------
if { $DO_SDC } {

    # Contraintes de timing
    read_sdc $::env(CONST_DIR)/timing.sdc

    # Vérification des contraintes de timing
    report_timing -lint > $::env(SYN_REP_DIR)/${DESIGN}.syn.timing_lint.rpt
}

#-----------------------------------------------------------------------------
# SYNTHESE
#-----------------------------------------------------------------------------
if { $DO_SYN } {

    # Ungroup
    ungroup -all -simple

    # Synthèse générique
    set_db / .syn_generic_effort ${SYN_EFFORT}
    syn_generic $DESIGN

    # Mapping des cellules standards
    set_db / .syn_map_effort ${SYN_EFFORT}
    syn_map $DESIGN

    # Optimisations
    set_db / .syn_opt_effort ${SYN_EFFORT}
    syn_opt $DESIGN
}

#-----------------------------------------------------------------------------
# RAPPORTS & NETLIST
#-----------------------------------------------------------------------------
if { $DO_REPORTS } {

    # Rapports
    report_timing > $::env(SYN_REP_DIR)/${DESIGN}.syn.timing.rpt
    report_area   > $::env(SYN_REP_DIR)/${DESIGN}.syn.area.rpt
    report_gates  > $::env(SYN_REP_DIR)/${DESIGN}.syn.gates.rpt
    report_power  > $::env(SYN_REP_DIR)/${DESIGN}.syn.power.rpt

    # Netlist
    write_hdl > $::env(SYN_NET_DIR)/${DESIGN}.syn.v
    write_sdf > $::env(SYN_NET_DIR)/${DESIGN}.syn.sdf
    write_sdc > $::env(CONST_DIR)/${DESIGN}.syn.sdc
}
